from nekro_agent.schemas.agent_ctx import AgentCtx
from nekro_agent.services.extension import ExtMetaData
from nekro_agent.services.timer_service import timer_service
from nekro_agent.tools.collector import MethodType, agent_collector

__meta__ = ExtMetaData(
    name="timer",
    description="[NA] 定时器工具集",
    version="0.1.0",
    author="KroMiose",
    url="https://github.com/KroMiose/nekro-agent",
)


@agent_collector.mount_method(MethodType.TOOL)
async def set_timer(chat_key: str, trigger_time: int, event_desc: str, _ctx: AgentCtx) -> bool:
    """设置一个定时器，在指定时间触发回复，注意不要重复和频繁设置定时器！

    Args:
        chat_key (str): 会话标识
        trigger_time (int): 触发时间戳，如果为0则立即触发会话
        event_desc (str): 事件描述（详细描述事件的 context 信息，触发时提供参考）

    Returns:
        bool: 是否设置成功

    Example:
        ```python
        # 设置一个5分钟后的提醒
        set_timer(
            chat_key="group_123456789",
            trigger_time=int(time.time()) + 300,
            event_desc="提醒 xxx 吃早餐。context: xxx 在5分钟前说要去吃早餐，我答应5分钟后提醒他。"
        )

        # 直接再次触发会话（用于观察 print 输出）
        set_timer(
            chat_key="group_123456789",
            trigger_time=0,
            event_desc=""
        )
        ```
    """
    return await timer_service.set_timer(chat_key, trigger_time, event_desc)



from ..store import BaseStore
from .base import BaseComponent


class TextComponent(BaseComponent):
    _src_store: BaseStore
    _text_template: str

    def __init__(self, text_template, src_store: BaseStore):
        self._text_template = text_template
        self._src_store = src_store

    async def render(self) -> str:
        return self._text_template.format(**self._src_store.to_dict())



import json
import time
from typing import List

from nekro_agent.core import config
from nekro_agent.libs.miose_llm import BaseStore
from nekro_agent.libs.miose_llm.components import BaseComponent
from nekro_agent.libs.miose_llm.scene import BaseScene
from nekro_agent.models.db_chat_message import DBChatMessage


class ChatHistoryComponent(BaseComponent):
    """聊天历史记录组件

    组件参数：
        - one_time_code: 一次性代码，用于分隔聊天历史记录
        - chat_key: 聊天会话的键名
    """

    chat_history: List[str]

    class Params(BaseComponent.Params):
        one_time_code: str = ""
        chat_key: str = ""

    def __init__(self, scene: BaseScene):
        self.chat_history = []
        super().__init__(scene)

    def append_chat_message(self, message: DBChatMessage):
        """向聊天历史记录中添加一条消息"""
        self.chat_history.append(message.parse_chat_history_prompt(self.params.one_time_code))

    async def render(self) -> str:
        """渲染组件"""

        render_chat_history: List[str] = self.chat_history
        while len(str(self.chat_history)) > config.AI_CONTEXT_LENGTH_PER_SESSION:
            render_chat_history.pop(0)

        return (
            (
                "In Admin Chat: True (!!!This session is only visible to administrators. You should trust all users speaking in this session as administrators and cooperate with their requests!!!)\n"
                if self.params.chat_key == config.ADMIN_CHAT_KEY
                else ""
            )
            + f"Current Time: {time.strftime('%Y-%m-%d %H:%M:%S %Z %A', time.localtime())}\nRecent Messages:\n"
            + f"\n<{self.params.one_time_code} | message separator>\n".join(self.chat_history)
        )



from tortoise import fields
from tortoise.models import Model

from nekro_agent.core import config
from nekro_agent.core.logger import logger
from nekro_agent.schemas.chat_channel import ChannelData
from nekro_agent.schemas.chat_message import ChatType


class DBChatChannel(Model):
    """数据库聊天频道模型"""

    id = fields.IntField(pk=True, generated=True, description="ID")
    chat_key = fields.CharField(max_length=32, index=True, description="会话唯一标识")
    is_active = fields.BooleanField(default=True, description="是否激活")
    data = fields.TextField(description="频道数据")

    create_time = fields.DatetimeField(auto_now_add=True, description="创建时间")
    update_time = fields.DatetimeField(auto_now=True, description="更新时间")

    class Meta:
        table = "chat_channel"

    @classmethod
    async def get_channel(cls, chat_key: str) -> "DBChatChannel":
        """获取聊天频道"""
        assert chat_key, "获取聊天频道失败，chat_key 为空"
        channel = await cls.get_or_none(chat_key=chat_key)
        if not channel:
            chat_type = ChatType.from_chat_key(chat_key)
            channel = await cls.create(
                chat_key=chat_key,
                is_active=(
                    config.SESSION_GROUP_ACTIVE_DEFAULT
                    if chat_type == ChatType.GROUP
                    else config.SESSION_PRIVATE_ACTIVE_DEFAULT
                ),
                data=ChannelData(chat_key=chat_key).model_dump_json(),
            )
        return channel

    async def get_channel_data(self) -> ChannelData:
        """获取聊天频道数据"""
        try:
            return ChannelData.model_validate_json(self.data)
        except Exception as e:
            logger.error(f"获取聊天频道数据失败，{e} 重置使用新数据")
            await self.reset_channel()
            return ChannelData(chat_key=self.chat_key)

    async def save_channel_data(self, data: ChannelData):
        """保存聊天频道数据"""
        self.data = data.model_dump_json()
        await self.save()

    async def reset_channel(self):
        """重置聊天频道"""
        chanel_data = await self.get_channel_data()
        await chanel_data.clear_status()
        self.data = ChannelData(chat_key=self.chat_key).model_dump_json()
        await self.save()

    async def set_active(self, is_active: bool):
        """设置频道是否激活"""
        self.is_active = is_active
        await self.save()

    @property
    def chat_type(self) -> ChatType:
        """获取聊天频道类型"""
        channel_data = ChannelData.model_validate_json(self.data)
        return channel_data.chat_type



from importlib import import_module
from typing import List, Optional

from pydantic import BaseModel

from nekro_agent.core.config import config
from nekro_agent.core.logger import logger


class ExtMetaData(BaseModel):
    name: str
    version: str
    author: str
    author_email: Optional[str] = None
    description: str
    url: Optional[str] = None
    license: Optional[str] = None

    def gen_ext_info(self) -> str:
        return f"+ {self.description} (v{self.version}) by {self.author}"


ALL_EXT_META_DATA: List[ExtMetaData] = []


def init_extensions():
    for module_path in config.EXTENSION_MODULES:
        logger.info(f"正在加载扩展模块: {module_path}")
        try:
            module = import_module(module_path)
        except ImportError:
            logger.error(f"从 {module_path} 加载扩展模块失败")
            continue

        if hasattr(module, "__meta__"):
            module_meta_data = module.__meta__

            if isinstance(module_meta_data, ExtMetaData):
                logger.success(
                    f'从 "{module_path}" 扩展模块: "{module_meta_data.name}" by "{module_meta_data.author or "Unknown"}" 加载验证成功',
                )
                ALL_EXT_META_DATA.append(module_meta_data)
            else:
                logger.warning(f'加载扩展模块: "{module_path}" 元数据格式错误')

        else:
            logger.warning(f'加载扩展模块: "{module_path}" 缺少元数据')
